<?php

use Axepta\Payment\Util;
use JsonSchema\Validator;

/**
 * Class ControllerExtensionPaymentAxepta
 */
class ControllerExtensionPaymentAxepta extends Controller
{
	private $error = [];

	/**
	 * view admin panel
	 */
	public function index()
	{
		$this->language->load('extension/payment/axepta');
		$this->document->setTitle($this->language->get('heading_title'));
		$this->document->addScript('view/javascript/select2/js/select2.min.js');
		$this->document->addStyle('view/javascript/select2/css/select2.css');
		$this->load->model('setting/setting');

		$request = $this->request->post;

		include_once(DIR_SYSTEM . 'library/payment-core/vendor/autoload.php');

		if(($this->request->server['REQUEST_METHOD'] == Util::METHOD_REQUEST_POST)
			&& $this->validate($request)
		) {

			$settings = array_merge($this->model_setting_setting->getSetting('payment_axepta'), $request);
			$this->model_setting_setting->editSetting('payment_axepta', $settings);
			$this->session->data['success'] = $this->language->get('text_success');
			$this->response->redirect($this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true));

			return;
		}

		$data = [

			'axepta_status' => isset($this->request->post['payment_axepta_status'])
				? $this->request->post['payment_axepta_status']
				: $this->config->get('payment_axepta_status'),

			'axepta_sandbox' => isset($this->request->post['payment_axepta_sandbox'])
				? $this->request->post['payment_axepta_sandbox']
				: $this->config->get('payment_axepta_sandbox'),

			'axepta_merchant_id' => isset($this->request->post['payment_axepta_merchant_id'])
				? $this->request->post['payment_axepta_merchant_id']
				: $this->config->get('payment_axepta_merchant_id'),

			'axepta_service_id' => isset($this->request->post['payment_axepta_service_id'])
				? $this->request->post['payment_axepta_service_id']
				: $this->config->get('payment_axepta_service_id'),

			'axepta_service_key' => isset($this->request->post['payment_axepta_service_key'])
				? $this->request->post['payment_axepta_service_key']
				: $this->config->get('payment_axepta_service_key'),

			'axepta_credit_type' => isset($this->request->post['payment_axepta_credit_type'])
				? $this->request->post['payment_axepta_credit_type']
				: $this->config->get('payment_axepta_credit_type'),

			'axepta_sort_order' => isset($this->request->post['payment_axepta_sort_order'])
				? $this->request->post['payment_axepta_sort_order']
				: $this->config->get('payment_axepta_sort_order'),

			'axepta_send_cart' => isset($this->request->post['payment_axepta_send_cart'])
				? $this->request->post['payment_axepta_send_cart']
				: $this->config->get('payment_axepta_send_cart'),

			'axepta_order_history' => isset($this->request->post['payment_axepta_order_history'])
				? $this->request->post['payment_axepta_order_history']
				: $this->config->get('payment_axepta_order_history'),

			'axepta_payment_title_default' => $this->language->get('text_axepta_payment_title_default'),
			'axepta_payment_title'         => isset($this->request->post['payment_axepta_payment_title'])
				? $this->request->post['payment_axepta_payment_title']
				: $this->config->get('payment_axepta_payment_title'),

			'axepta_all_currencies' => Util::getSupportedCurrencies(),
			'axepta_currencies'     => isset($this->request->post['payment_axepta_currencies'])
				? $this->request->post['payment_axepta_currencies']
				: ($this->config->get('payment_axepta_currencies')
					?: []),

			'error_validation' => $this->error,

			'notification_url' => HTTPS_CATALOG . 'index.php?route=extension/payment/axepta/notification',

			'confirm_url' => $this->url->link('extension/payment/axepta', 'user_token=' . $this->session->data['user_token'], true),
			'cancel_url'  => $this->url->link('extension/extension', 'user_token=' . $this->session->data['user_token'], true),

			'content_footer'      => $this->load->controller('common/footer'),
			'content_header'      => $this->load->controller('common/header'),
			'content_column_left' => $this->load->controller('common/column_left'),
			'heading_title'       => $this->language->get('text_heading_title'),

			'breadcrumbs' => [
				[
					'text'      => $this->language->get('text_home'),
					'href'      => $this->url->link('common/home', 'user_token=' . $this->session->data['user_token'], 'SSL'),
					'separator' => false,
				],

				[
					'text'      => $this->language->get('text_payment'),
					'href'      => $this->url->link('extension/extension', 'user_token=' . $this->session->data['user_token'], 'SSL'),
					'separator' => ' :: ',
				],

				[
					'text'      => $this->language->get('heading_title'),
					'href'      => $this->url->link('extension/payment/axepta', 'user_token=' . $this->session->data['user_token'], 'SSL'),
					'separator' => ' :: ',
				],
			],

		];

		$this->response->setOutput($this->load->view('extension/payment/axepta', $data));

		$current_axepta_settings = $this->model_setting_setting->getSetting('payment_axepta');

		if(isset($current_axepta_settings['axepta_error_tmp'])) {
			unset($current_axepta_settings['axepta_error_tmp']);
			$this->model_setting_setting->editSetting('axepta', $current_axepta_settings);
		}

		if(isset($current_axepta_settings['axepta_success_tmp'])) {
			unset($current_axepta_settings['axepta_success_tmp']);
			$this->model_setting_setting->editSetting('axepta', $current_axepta_settings);
		}
	}

	/**
	 * @param array $request
	 *
	 * @return bool
	 */
	protected function validate($request)
	{
		if(!$this->user->hasPermission('modify', 'extension/payment/axepta')) {
			$this->error['error_warning_permissions'] = $this->language->get('error_permission');

			return false;
		}

		include_once(DIR_SYSTEM . 'library/payment-core/vendor/autoload.php');

		$schema = [
			'type'       => 'object',
			'properties' => [

				'payment_axepta_status'        => [
					'type' => 'string',
					"enum" => [
						'0',
						'1',
					],
				],
				'payment_axepta_sandbox'       => [
					'type' => 'string',
					"enum" => [
						'0',
						'1',
					],
				],
				'payment_axepta_merchant_id'   => [
					'type' => 'string',
				],
				'payment_axepta_service_id'    => [
					'type' => 'string',
				],
				'payment_axepta_service_key'   => [
					'type' => 'string',
				],
				'payment_axepta_credit_type'   => [
					'type' => 'string',
				],
				'payment_axepta_sort_order'    => [
					'type'      => 'string',
					'minLength' => 1,
				],
				'payment_axepta_payment_title' => [
					'type' => 'string',
				],
				'payment_axepta_currencies'    => [
					'type'        => 'array',
					'items'       => [
						'type' => 'string',
						'enum' => array_values(Util::getSupportedCurrencies()),
					],
					'minItems'    => 1,
					'uniqueItems' => true,
				],
			],

			'required'             => [
				'payment_axepta_status',
				'payment_axepta_sandbox',
				'payment_axepta_merchant_id',
				'payment_axepta_service_id',
				'payment_axepta_service_key',
				'payment_axepta_sort_order',
				'payment_axepta_payment_title',
				'payment_axepta_currencies',
			],
			'additionalProperties' => false,
		];

		$data = json_decode(json_encode($request));

		$validator = new Validator();
		$validator->validate($data, $schema);

		$this->error = [];

		foreach($validator->getErrors() as $key => $value) {

			if($value['constraint'] === 'additionalProp') {
				$this->error['additional_properties'] = true;
			} else {
				$this->error[$value['property']] = true;
			}
		}

		return empty($this->error);
	}
}
